@@ -1,3 +1,17 @@
+#   Copyright ETH 2018 - 2023 Zürich, Scientific IT Services
+# 
+#   Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
+# 
+#        http://www.apache.org/licenses/LICENSE-2.0
+#   
+#   Unless required by applicable law or agreed to in writing, software
+#   distributed under the License is distributed on an "AS IS" BASIS,
+#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#   See the License for the specific language governing permissions and
+#   limitations under the License.
+#
 import os
 from functools import partialmethod
 from pathlib import Path
@@ -35,6 +49,16 @@ dataset_definitions = openbis_definitions("dataSet")
 dss_endpoint = "/datastore_server/rmi-data-store-server-v3.json"
 
 
+def signed_to_unsigned(sig_int):
+    """openBIS delivers crc32 checksums as signed integers.
+    If the number is negative, we just have to add 2**32
+    We display the hex number to match with the classic UI
+    """
+    if sig_int < 0:
+        sig_int += 2**32
+    return "%x" % (sig_int & 0xFFFFFFFF)
+
+
 class DataSet(
     OpenBisObject,
     entity="dataSet",
@@ -422,6 +446,12 @@ class DataSet(
                 dataSetFiles["downloadUrl"] = dataSetFiles["dataStore"].map(
                     extract_downloadUrl
                 )
+                dataSetFiles["checksumCRC32"] = (
+                    dataSetFiles["checksumCRC32"]
+                    .fillna(0.0)
+                    .astype(int)
+                    .map(signed_to_unsigned)
+                )
                 dataSetFiles["dataStore"] = dataSetFiles["dataStore"].map(extract_code)
                 dataSetFiles["dataSetPermId"] = dataSetFiles["dataSetPermId"].map(
                     extract_permid
@@ -513,8 +543,8 @@ class DataSet(
         with DataSetDownloadQueue(workers=workers) as queue:
             # get file list and start download
             for filename in files:
-                file_info = self.get_file_list(start_folder=filename)
-                file_size = file_info[0]["fileSize"]
+                fi_df = self.get_dataset_files().df
+                file_size = fi_df[fi_df["path"] == filename]["fileLength"].values[0]
                 download_url = base_url + filename + "?sessionID=" + self.openbis.token
                 download_url = quote(download_url, safe=":/?=")
                 filename_dest = ""
@@ -570,8 +600,8 @@ class DataSet(
             content_copy = self.data["linkedData"]["contentCopies"][content_copy_index]
 
             for filename in files:
-                file_info = self.get_file_list(start_folder=filename)
-                file_size = file_info[0]["fileSize"]
+                fi_df = self.get_dataset_files().df
+                file_size = fi_df[fi_df["path"] == filename]["fileLength"].values[0]
 
                 download_url = linked_dataset_fileservice_url
                 download_url += "?sessionToken=" + self.openbis.token
@@ -629,13 +659,8 @@ class DataSet(
         if self.is_new:
             return self.files
         else:
-            files = []
-            for file in self.get_file_list(recursive=True):
-                if file["isDirectory"]:
-                    pass
-                else:
-                    files.append(file["pathInDataSet"])
-            return files
+            fl = self.get_dataset_files().df
+            return fl[fl["directory"] == False]["path"].to_list()
 
     @property
     def file_links(self):
@@ -681,29 +706,21 @@ class DataSet(
 
     def get_files(self, start_folder="/"):
         """Returns a DataFrame of all files in this dataset"""
-
-        def createRelativePath(pathInDataSet):
-            if self.shareId is None:
-                return ""
-            else:
-                return os.path.join(self.shareId, self.location, pathInDataSet)
-
-        def signed_to_unsigned(sig_int):
-            """openBIS delivers crc32 checksums as signed integers.
-            If the number is negative, we just have to add 2**32
-            We display the hex number to match with the classic UI
-            """
-            if sig_int < 0:
-                sig_int += 2**32
-            return "%x" % (sig_int & 0xFFFFFFFF)
-
-        files = self.get_file_list(start_folder=start_folder)
-        df = DataFrame(files)
-        df["relativePath"] = df["pathInDataSet"].map(createRelativePath)
-        df["crc32Checksum"] = (
-            df["crc32Checksum"].fillna(0.0).astype(int).map(signed_to_unsigned)
+        if start_folder.startswith("/"):
+            start_folder = start_folder[1:]
+        file_list = self.get_dataset_files().df
+        file_list[file_list["path"].str.startswith(start_folder)]
+        new_file_list = file_list[
+            ["directory", "path", "fileLength", "checksumCRC32"]
+        ].rename(
+            columns={
+                "directory": "isDirectory",
+                "path": "pathInDataSet",
+                "fileLength": "fileSize",
+                "checksumCRC32": "crc32Checksum",
+            }
         )
-        return df[["isDirectory", "pathInDataSet", "fileSize", "crc32Checksum"]]
+        return new_file_list
 
     def _get_download_url(self):
         download_url = ""
@@ -720,6 +737,7 @@ class DataSet(
         By default, all directories and their containing files are listed recursively. You can
         turn off this option by setting recursive=False.
         """
+        print("This method is deprecated. Consider using get_files() instead")
         request = {
             "method": "listFilesForDataSet",
             "params": [
