@@ -1,3 +1,17 @@
+#   Copyright ETH 2018 - 2023 Zürich, Scientific IT Services
+# 
+#   Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
+# 
+#        http://www.apache.org/licenses/LICENSE-2.0
+#   
+#   Unless required by applicable law or agreed to in writing, software
+#   distributed under the License is distributed on an "AS IS" BASIS,
+#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#   See the License for the specific language governing permissions and
+#   limitations under the License.
+#
 from .attribute import AttrHolder
 from .definitions import openbis_definitions
 from .openbis_object import OpenBisObject
@@ -9,7 +23,7 @@ class Sample(OpenBisObject, entity="sample", single_item_method_name="get_sample
     """A Sample (new: Object) is one of the most commonly used entities in openBIS."""
 
     def __init__(
-        self, openbis_obj, type, project=None, data=None, props=None, **kwargs
+        self, openbis_obj, type, project=None, data=None, props=None, attrs=None, **kwargs
     ):
         self.__dict__["openbis"] = openbis_obj
         self.__dict__["type"] = type
@@ -21,15 +35,6 @@ class Sample(OpenBisObject, entity="sample", single_item_method_name="get_sample
         if data is not None:
             self._set_data(data)
 
-        # TODO: Why are we using getattr() and setattr() here? They are considerably slower.
-        if project is not None:
-            self.project = project
-
-        if props is not None:
-            for key in props:
-                # self.p[key] = props[key]
-                setattr(self.p, key, props[key])
-
         if kwargs is not None:
             for key in kwargs:
                 setattr(self, key, kwargs[key])
@@ -43,17 +48,34 @@ class Sample(OpenBisObject, entity="sample", single_item_method_name="get_sample
                 except Exception:
                     pass
 
+        if project is None:
+            if self.experiment:
+                self.project = self.experiment.project
+        else:
+            self.project = project
+
+        if props is not None:
+            for key in props:
+                # self.p[key] = props[key]
+                setattr(self.p, key, props[key])
+
         if getattr(self, "parents") is None:
             self.a.__dict__["_parents"] = []
         else:
             if not self.is_new:
-                self.a.__dict__["_parents_orig"] = self.a.__dict__["_parents"]
+                if attrs is not None and "parents" not in attrs:
+                    self.a.__dict__["_parents"] = None
+                else:
+                    self.a.__dict__["_parents_orig"] = self.a.__dict__["_parents"]
 
         if getattr(self, "children") is None:
             self.a.__dict__["_children"] = []
         else:
             if not self.is_new:
-                self.a.__dict__["_children_orig"] = self.a.__dict__["_children"]
+                if attrs is not None and "children" not in attrs:
+                    self.a.__dict__["_children"] = None
+                else:
+                    self.a.__dict__["_children_orig"] = self.a.__dict__["_children"]
 
         if getattr(self, "components") is None:
             self.a.__dict__["_components"] = []
@@ -209,9 +231,7 @@ class Sample(OpenBisObject, entity="sample", single_item_method_name="get_sample
                             or getattr(self.props, prop_name) == ""
                         ):
                             raise ValueError(
-                                "Property '{}' is mandatory and must not be None".format(
-                                    prop_name
-                                )
+                                f"Property '{prop_name}' is mandatory and must not be None"
                             )
 
             sampleProject = self.project.code if self.project else None
@@ -246,7 +266,7 @@ class Sample(OpenBisObject, entity="sample", single_item_method_name="get_sample
                 if resp["rows"][0][0]["value"] != "OK":
                     raise ValueError("Status is not OK")
                 if VERBOSE:
-                    print("{} successfully created.".format(self.entity))
+                    print(f"{self.entity} successfully created.")
                 permId = permid = resp["rows"][0][2]["value"]
                 new_entity_data = self.openbis.get_sample(permId, only_data=True)
                 self._set_data(new_entity_data)
