@@ -1,8 +1,23 @@
-from tabulate import tabulate
-from pandas import DataFrame, Series
+#   Copyright ETH 2018 - 2023 Zürich, Scientific IT Services
+# 
+#   Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
+# 
+#        http://www.apache.org/licenses/LICENSE-2.0
+#   
+#   Unless required by applicable law or agreed to in writing, software
+#   distributed under the License is distributed on an "AS IS" BASIS,
+#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#   See the License for the specific language governing permissions and
+#   limitations under the License.
+#
 import pandas as pd
+from pandas import DataFrame
+from tabulate import tabulate
 
-class Things():
+
+class Things:
     """An object that contains a DataFrame object about an entity  available in openBIS.
     Use .df to work with the DataFrame instead.
     Can be used in a for-loop:
@@ -23,15 +38,20 @@ class Things():
     """
 
     def __init__(
-        self, openbis_obj, entity,
-        identifier_name='code', additional_identifier=None,
-        start_with=None, count=None, totalCount=None,
-        single_item_method=None,
-        response=None,
-        df_initializer=None,
-        objects_initializer=None,
-        attrs=None,
-        props=None
+            self,
+            openbis_obj,
+            entity,
+            identifier_name="code",
+            additional_identifier=None,
+            start_with=None,
+            count=None,
+            totalCount=None,
+            single_item_method=None,
+            response=None,
+            df_initializer=None,
+            objects_initializer=None,
+            attrs=None,
+            props=None,
     ):
         self.openbis = openbis_obj
         self.entity = entity
@@ -40,8 +60,8 @@ class Things():
         self.additional_identifier = additional_identifier
         self.start_with = start_with
         self.count = count
-        self.totalCount=totalCount
-        self.single_item_method=single_item_method
+        self.totalCount = totalCount
+        self.single_item_method = single_item_method
         self.__objects = None
         self.response = response
         self.__objects_initializer = objects_initializer
@@ -58,7 +78,9 @@ class Things():
     @property
     def df(self):
         if self.__df is None and self.__df_initializer is not None:
-            self.__df = self.__df_initializer(attrs=self.__attrs, props=self.__props, response=self.response)
+            self.__df = self.__df_initializer(
+                attrs=self.__attrs, props=self.__props, response=self.response
+            )
         return self.__df
 
     @property
@@ -67,8 +89,12 @@ class Things():
             self.__objects = self.__objects_initializer(response=self.response)
         return self.__objects
 
-    def __repr__(self):
-        return tabulate(self.df, headers=list(self.df))
+    def __repr__(self, headers=None, sort_by=None):
+        if headers is None:
+            headers = list(self.df)
+        if sort_by:
+            return tabulate(self.df.sort_values(by=sort_by), headers=headers)
+        return tabulate(self.df, headers=headers)
 
     def __len__(self):
         return len(self.df)
@@ -84,42 +110,55 @@ class Things():
             return DataFrame()
 
     def get_parents(self, **kwargs):
-        if self.entity not in ['sample', 'dataset']:
-            raise ValueError("{}s do not have parents".format(self.entity))
+        if self.entity not in ["sample", "dataset"]:
+            raise ValueError(f"{self.entity}s do not have parents")
 
         if self.df is not None and len(self.df) > 0:
             dfs = []
             for ident in self.df[self.identifier_name]:
                 # get all objects that have this object as a child == parent
                 try:
-                    parents = getattr(self.openbis, 'get_' + self.entity.lower() + 's')(withChildren=ident, **kwargs)
+                    parents = getattr(self.openbis, "get_" + self.entity.lower() + "s")(
+                        withChildren=ident, **kwargs
+                    )
                     dfs.append(parents.df)
                 except ValueError:
                     pass
-            return Things(self.openbis, self.entity, self.identifier_name, response=dfs,
-                          df_initializer=self.__create_data_frame)
-
+            return Things(
+                self.openbis,
+                self.entity,
+                self.identifier_name,
+                response=dfs,
+                df_initializer=self.__create_data_frame,
+            )
 
     def get_children(self, **kwargs):
-        if self.entity not in ['sample', 'dataset']:
-            raise ValueError("{}s do not have children".format(self.entity))
+        if self.entity not in ["sample", "dataset"]:
+            raise ValueError(f"{self.entity}s do not have children")
 
         if self.df is not None and len(self.df) > 0:
             dfs = []
             for ident in self.df[self.identifier_name]:
                 # get all objects that have this object as a child == parent
                 try:
-                    parents = getattr(self.openbis, 'get_' + self.entity.lower() + 's')(withParent=ident, **kwargs)
+                    parents = getattr(self.openbis, "get_" + self.entity.lower() + "s")(
+                        withParent=ident, **kwargs
+                    )
                     dfs.append(parents.df)
                 except ValueError:
                     pass
 
-                return Things(self.openbis, self.entity, self.identifier_name, response=dfs,
-                              df_initializer=self.__create_data_frame)
+                return Things(
+                    self.openbis,
+                    self.entity,
+                    self.identifier_name,
+                    response=dfs,
+                    df_initializer=self.__create_data_frame,
+                )
 
     def get_samples(self, **kwargs):
-        if self.entity not in ['space', 'project', 'experiment']:
-            raise ValueError("{}s do not have samples".format(self.entity))
+        if self.entity not in ["space", "project", "experiment"]:
+            raise ValueError(f"{self.entity}s do not have samples")
 
         if self.df is not None and len(self.df) > 0:
             dfs = []
@@ -132,14 +171,19 @@ class Things():
                 except ValueError:
                     pass
 
-            return Things(self.openbis, 'sample', 'identifier', response=dfs,
-                          df_initializer=self.__create_data_frame)
+            return Things(
+                self.openbis,
+                "sample",
+                "identifier",
+                response=dfs,
+                df_initializer=self.__create_data_frame,
+            )
 
-    get_objects = get_samples # Alias
+    get_objects = get_samples  # Alias
 
     def get_datasets(self, **kwargs):
-        if self.entity not in ['sample', 'experiment']:
-            raise ValueError("{}s do not have datasets".format(self.entity))
+        if self.entity not in ["sample", "experiment"]:
+            raise ValueError(f"{self.entity}s do not have datasets")
 
         if self.df is not None and len(self.df) > 0:
             dfs = []
@@ -152,11 +196,16 @@ class Things():
                 except ValueError:
                     pass
 
-            return Things(self.openbis, 'dataset', 'permId', response=dfs,
-                          df_initializer=self.__create_data_frame)
+            return Things(
+                self.openbis,
+                "dataset",
+                "permId",
+                response=dfs,
+                df_initializer=self.__create_data_frame,
+            )
 
     def __getitem__(self, key):
-        """ elegant way to fetch a certain element from the displayed list.
+        """elegant way to fetch a certain element from the displayed list.
         If an integer value is given, we choose the row.
         If the key is a list, we return the desired columns (normal dataframe behaviour)
         If the key is a non-integer value, we treat it as a primary-key lookup
@@ -181,14 +230,14 @@ class Things():
                 if self.single_item_method:
                     get_item = self.single_item_method
                 else:
-                    get_item = getattr(self.openbis, 'get_' + self.entity)
+                    get_item = getattr(self.openbis, "get_" + self.entity)
                 if self.additional_identifier is None:
                     return get_item(row[self.identifier_name].values[0])
                 ## get an entry using two keys
                 else:
                     return get_item(
-                            row[self.identifier_name].values[0],
-                            row[self.additional_identifier].values[0]
+                        row[self.identifier_name].values[0],
+                        row[self.additional_identifier].values[0],
                     )
 
     def __iter__(self):
@@ -199,10 +248,8 @@ class Things():
             if self.single_item_method:
                 get_item = self.single_item_method
             else:
-                get_item = getattr(self.openbis, 'get_' + self.entity)
-            for item in self.df[[self.identifier_name]][self.identifier_name].iteritems():
+                get_item = getattr(self.openbis, "get_" + self.entity)
+            for item in self.df[[self.identifier_name]][
+                self.identifier_name
+            ].items():
                 yield get_item(item[1])
-                #yield getattr(self.openbis, 'get_' + self.entity)(item[1])
-
-                # return self.df[[self.identifier_name]].to_dict()[self.identifier_name]
-
